%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% This function estimates the threshold and the slope coefficients in
% standard theshold regression
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [qhat,beta1,beta2,se1,se2,qhat1,qhat2] = est_threshold(X,Y,q,N,h,confi)
n               = length(X);
T               = n/N;
k               = size(X,2);

[q,index]       = sort(q);
Y               = Y(index);
X               = X(index,:);   
%% overall estimation
mi              = inv(X'*X);
beta            = mi*(X'*Y);
e               = Y-X*beta;
ee              = e'*e;
sig             = ee/(n-k);
xe              = X.*(e*ones(1,length(X(1,:))));
% if h == 0
%     se          = sqrt(diag(mi)*sig);
% else
%     se          = sqrt(diag(mi*xe'*xe*mi));
% end
% vy              = sum((Y-mean(Y)').*(Y-mean(Y)'))';

%% threshold estimation
qs              = unique(q);
qn              = length(qs);
sn              = zeros(qn,1);

irb             = zeros(n,1);
mm              = zeros(k,k);
sume            = zeros(k,1);
ci              = 0;

r               = 1;
while r <= qn
    irf         = ( q <= qs(r) );
    ir          = irf-irb;
    irb         = irf;
    ci          = ci+sum(ir)';
    xir         = X(ci,:);
    mm          = mm + xir'*xir;
    sume        = sume + (ir'*xe)';
    mmi         = mm - mm*mi*mm;
    if (ci>k+1)*(ci<(n-k-1))
        sn(r)   = ee - sume'*(inv(mmi)*sume);
    else
        sn(r)   = ee;
    end
    r           = r+1;
end
[smin, minidx]  = min(sn);
smin            = sn(minidx);
sighat          = smin/n;
qhat            = qs(minidx);

%% split-sample estimation
i1              = (q<=qhat);    % indicator of the first subsample
i2              = ~(i1);        % indicator of the second subsample

x1              = X(i1,:);
y1              = Y(i1,:);
x2              = X(i2,:);
y2              = Y(i2,:);

mi1             = inv(x1'*x1);
mi2             = inv(x2'*x2);
beta1           = mi1*(x1'*y1);
beta2           = mi2*(x2'*y2);
e1              = y1-x1*beta1;
e2              = y2-x2*beta2;
eful            = [e1;e2];
n1              = length(y1(:,1));
n2              = length(y2(:,1));
ee1             = e1'*e1;
ee2             = e2'*e2;
sig1            = ee1/(n1-k);
sig2            = ee2/(n2-k);
sigful          = (ee1+ee2)/(n-2*k);
if h == 0
    se1         = sqrt(diag(mi1)*sigful);
    se2         = sqrt(diag(mi2)*sigful);
elseif h == 1
    se1         = sqrt(diag(mi1*(x1.*(e1*(ones(1,length(x1(1,:))))))'*(x1.*(e1*(ones(1,length(x1(1,:))))))*mi1));
    se2         = sqrt(diag(mi2*(x2.*(e2*(ones(1,length(x2(1,:))))))'*(x2.*(e2*(ones(1,length(x2(1,:))))))*mi2));
elseif h == 2   
    [~,se1,~]   = hac(x1,y1,'intercept',false,'display','off','bandwidth',0.9);
    [~,se2,~]   = hac(x2,y2,'intercept',false,'display','off','bandwidth',0.9);    
end

%% confidence interval of threshold 
if h == 0
    lr          = (sn-smin)/sighat;
else
    nonpar_     = 2;
    r1          = (X*(beta1-beta2)).*(X*(beta1-beta2));
    r2          = r1.*((eful.*eful)*ones(1,length(r1(1,:))));
    qx          = [q.^0,q.^1,q.^2];
    qh          = [qhat.^0,qhat.^1,qhat.^2];
    m1          = (r1'/qx')';
    m2          = (r2'/qx')';
    g1          = qh*m1;
    g2          = qh*m2;
    if nonpar_== 2
        sigq    = sqrt(mean((q-mean(q)').^2));
        hband   = 2.1*sigq/(n^(.25));
        u       = (qhat-q)/hband;
        u2      = u.^2;
        f       = mean((1-u2).*(u2<=1))'*(1.5/(hband^2));
        df      = -mean((1-u2).*(u2<=1))'*(1.5/(hband^2));
        eps     = r1-qx*m1;
        sige    = (eps'*eps)/(n-3);
        hband   = sige/(4*f*((m1(3)+(m1(2)+2*m1(3)*qhat)*df/f)^2));
        u2      = ((qhat-q)/hband).^2;
        kh      = ((1-u2)*.75/hband).*(u2<=1);
        g1      = mean(kh.*(r1*ones(1,length(kh(1,:)))))';
        g2      = mean(kh.*(r2*ones(1,length(kh(1,:)))))';
    end
    eta2        = g2/g1;
    lr          = (sn-smin)/eta2;
end
c1              = -2*log(1-sqrt(confi));
lr1             = (lr>=c1);
if max(lr1)==1
    minind_lr1  = 0;
    for i = 1:length(lr1)
        if lr1(i)==min(lr1)
            minind_lr1 = i;
            break;
        end
    end
    qhat1       = qs(minind_lr1);
    minind_lr1v = 0;
    for i = 1:length(lr1)
        if lr1(qn+1-i)==min(lr1)
            minind_lr1v = i;
            break;
        end
    end
    qhat2       = qs(qn+1-minind_lr1v);
else
    qhat1       = qs(1);
    qhat2       = qs(qn);
end

qhat1           = qhat1';
qhat2           = qhat2';
end
