%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% This is the main file for the empirical analysis 
%
% Note that the replication results may be slightly different due to the
% randomization of initial values
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

clear
clc
warning off;
%%
% bank_reg    = xlsread('bank_regulation_data','drop_missing');               % when threshold variable is small firms and small banks
bank_reg    = xlsread('bank_regulation_data','full_data');                  % when threshold variable is dropout and unemp
TT          = 31;
NT          = length(bank_reg(:,1));
N           = NT/TT;

regu        = bank_reg(:,1);
popdisp     = bank_reg(:,2);
dropout     = bank_reg(:,3); 
smallbank   = bank_reg(:,4);
smallfirm   = bank_reg(:,5);
id          = bank_reg(:,6);                                                 
unemp       = bank_reg(:,7);
unitbank    = bank_reg(:,8);
year        = bank_reg(:,9);                                                  
logitgini   = bank_reg(:,10); 
log_gini    = bank_reg(:,11);                                                   
growth      = bank_reg(:,12);
black       = bank_reg(:,13);
female      = bank_reg(:,14);

y           = logitgini;
x           = [regu dropout unemp];
% x           = [regu dropout unemp growth black female];

thresh      = dropout;
k           = size(x,2);

%% within transformation
T               = TT;
Ry              = reshape(y,T,N);
ydm             = Ry - repmat(mean(Ry),T,1);
for j = 1:k
    Rx          = reshape(x(:,j),T,N);
    Rxdm        = Rx - repmat(mean(Rx),T,1);
    xdm(:,j)    = reshape(Rxdm,N*T,1);
end
Y               = reshape(ydm,N*T,1);
X               = reshape(xdm,N*T,k);   

% Y               = y;
% X               = x;
%% estimation    
h               = 2;
time            = repmat(1:T,1,N)';        
code            = kron((1:N)',ones(T,1)); 
confi           = 0.95;
idxdata         = dataset(code, time, Y, X, thresh);        
idxdata.Properties.VarNames = {'N'  'T'  'y'  'X' 'q'};     
G               = 2;
alpha           = 0.90;
[pmax,psum]     = test_existence_threshold(Y,X,thresh,N,G,h)
[pcom]          = test_common_threshold(idxdata,G,h,alpha)
[theta1,theta2,gamma,gcf1,gcf2,group,se1,se2,resQ] = est_pstr_kmeans(idxdata,G,h,confi);
[theta1,theta2,gamma,gcf1,gcf2,group,se1,se2,resQ] = est_pstr_comthd(idxdata,G,h,confi);

t1              = theta1./se1;
t2              = theta2./se2;

clearvars -except estG_IC1 estG_IC2 IC1 IC2 theta1 theta2 se1 se2 t1 t2 gamma group G pmax psum
filename = ['Est_thesh_var = dropout'];
save(filename)

